@@ -10,9 +10,19 @@
 #include "kontactinterface_debug.h"
 
 #include <KAboutData>
-#include <KStartupInfo>
 #include <KWindowSystem>
 
+#include "config-kontactinterface.h"
+#if KONTACTINTERFACE_HAVE_X11
+#include <KStartupInfo>
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
+#include <QX11Info>
+#endif
+#endif
+
 #include <QCommandLineParser>
 #include <QDir>
 
@@ -50,7 +60,7 @@ public:
         // Disable Chromium's own crash handler, which overrides DrKonqi.
         auto flags = qgetenv(kChromiumFlagsEnv);
         if (!flags.contains(kDisableInProcessStackTraces)) {
-            qputenv(kChromiumFlagsEnv, flags + " " + kDisableInProcessStackTraces);
+            qputenv(kChromiumFlagsEnv, QByteArray(flags + " " + kDisableInProcessStackTraces));
         }
     }
 
@@ -98,7 +108,7 @@ static bool callNewInstance(const QString &appName, const QString &serviceName,
 
 int PimUniqueApplication::newInstance()
 {
-    return newInstance(KStartupInfo::startupId(), QStringList() << QApplication::applicationName(), QDir::currentPath());
+    return newInstance(QByteArray(), QStringList() << QApplication::applicationName(), QDir::currentPath());
 }
 
 bool PimUniqueApplication::start(const QStringList &arguments)
@@ -113,21 +123,13 @@ bool PimUniqueApplication::start(const QStringList &arguments)
     if (QDBusConnection::sessionBus().interface()->isServiceRegistered(serviceName)) {
         QByteArray new_asn_id;
         if (KWindowSystem::isPlatformX11()) {
-            KStartupInfoId id;
-            if (!KStartupInfo::startupId().isEmpty()) {
-                id.initId(KStartupInfo::startupId());
-            } else {
-                id = KStartupInfo::currentStartupIdEnv();
-            }
-            if (!id.isNull()) {
-                new_asn_id = id.id();
-            }
+#if KONTACTINTERFACE_HAVE_X11
+            new_asn_id = QX11Info::nextStartupId();
+#endif
         } else if (KWindowSystem::isPlatformWayland()) {
             new_asn_id = qgetenv("XDG_ACTIVATION_TOKEN");
         }
 
-        KWindowSystem::allowExternalProcessWindowActivation();
-
         if (callNewInstance(appName, serviceName, new_asn_id, arguments)) {
             return false; // success means that main() can exit now.
         }
@@ -149,7 +151,9 @@ bool PimUniqueApplication::start(const QStringList &arguments)
 int PimUniqueApplication::newInstance(const QByteArray &startupId, const QStringList &arguments, const QString &workingDirectory)
 {
     if (KWindowSystem::isPlatformX11()) {
+#if KONTACTINTERFACE_HAVE_X11
         KStartupInfo::setStartupId(startupId);
+#endif
     } else if (KWindowSystem::isPlatformWayland()) {
         KWindowSystem::setCurrentXdgActivationToken(QString::fromUtf8(startupId));
     }
@@ -159,11 +163,8 @@ int PimUniqueApplication::newInstance(const QByteArray &startupId, const QString
         if (qobject_cast<QMainWindow *>(win)) {
             win->show();
             win->setAttribute(Qt::WA_NativeWindow, true);
-            if (KWindowSystem::isPlatformX11()) {
-                KStartupInfo::setNewStartupId(win->windowHandle(), startupId); // this moves 'win' to the current desktop
-            } else if (KWindowSystem::isPlatformWayland()) {
-                KWindowSystem::activateWindow(win->windowHandle());
-            }
+
+            KWindowSystem::activateWindow(win->windowHandle());
             break;
         }
     }
